#[derive(Clone, Copy, Debug, PartialEq)]
enum Square {
    Empty,
    Cross,
    Circle,
}

impl std::fmt::Display for Square {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Square::Empty => write!(f, "."),
            Square::Cross => write!(f, "X"),
            Square::Circle => write!(f, "O"),
        }
    }
}

struct Board {
    board: [Square; 9],
}

impl Board {
    /// Creates a new empty board
    fn new() -> Self {
        Self {
            board: [Square::Empty; 9],
        }
    }

    /// Display the board to standard output
    fn display(&self) {
        println!("{}", self);
    }
}

impl std::fmt::Display for Board {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        for row in 0..3 {
            writeln!(f, " {} | {} | {}",
                    self.board[row * 3],
                    self.board[row * 3 + 1],
                    self.board[row * 3 + 2],
                )?;
            if row == 0 || row == 1 {
                writeln!(f, "---+---+---")?;
            }
        }
        Ok(())
    }
}

fn main() {
    let board = Board::new();

    board.display();
}

// Will only be build in 'test' configuration
#[cfg(test)]
// A module where the tests are located, more on modules later
mod tests {
    use super::*;

    #[test]
    fn should_pass() {
        assert!(true);
    }
    #[test]
    #[should_panic]
    fn should_panic() {
        panic!("Doing wrong things on purpose here!");
    }

    #[test]
    fn new_board_squares_are_empty() {
        let board = Board::new();

        for square in board.board {
            assert_eq!(square, Square::Empty);
        }
    }

    #[test]
    fn empty_board_to_string() {
        let expected = " . | . | .
---+---+---
 . | . | .
---+---+---
 . | . | .
";
        assert_eq!(expected, format!("{}", Board::new()));
    }

    #[test]
    fn empty_square_to_string_dot() {
        assert_eq!(".", format!("{}", Square::Empty));
    }
    #[test]
    fn cross_square_to_string_x() {
        assert_eq!("X", format!("{}", Square::Cross));
    }
    #[test]
    fn circle_square_to_string_big_o() {
        assert_eq!("O", format!("{}", Square::Circle));
    }
}
